// Project 57 - Analog Thermometer

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <math.h>

void startADC()
// Set up the ADC
{
    ADMUX |= (1 << REFS0);                 // Use AVcc pin with ADC
    ADMUX |= (1 << MUX2) | (1 << MUX0);    // use ADC5 (pin 28)
    ADCSRA |= (1 << ADPS2) |(1 << ADPS1) | (1 << ADPS0); // prescaler for 16MHz (/128)	
    ADCSRA |= (1 << ADEN);                 // enable ADC
}

void initPWM()
{    // activate PWM on PB1
    TCCR1A |= (1 << WGM11);
    TCCR1B |= (1 << WGM12)|(1 << WGM13)|(1 << CS11); 
	
    // connect PWM to PB1
    TCCR1A |= (1 << COM1A1)|(1 << COM1B1); // PWM to OCR1A - PB1 
    ICR1=39999; 
}

void servoAngle(uint8_t angle)
{    // rotate servo to 'angle' position
    OCR1A = ((angle-239.95)/-0.05); // convert angle to OCR1A (duty cycle) value
}

int main()
{
    DDRB|=(1<<PB1); // PORTB1 as output for servo control
    DDRC|=(0<<PC5); // PORTC5 as input for TMP36 measurement

    float temperature;
    float voltage;
    uint16_t ADCvalue;
    uint8_t finalAngle;
	
    startADC();
    initPWM();
	
    while(1)
    {
        // Take reading from TMP36 via ADC and determine temperature in Celsius
        ADCSRA |= (1 << ADSC);         // Start ADC measurement
        while (ADCSRA & (1 << ADSC) ); // Wait for conversion to finish 
    	_delay_ms(10);

        // Get value from ADC (which is 10-bit) register, store in ADCvalue
        ADCvalue = ADC;
	
        // convert reading to temperature value (Celsius)
        voltage = (ADCvalue * 5);
        voltage = voltage / 1024;
        temperature = ((voltage - 0.5) * 100);	
		
        // display temperature using servo
        finalAngle = 6 * temperature; // 0~30 degrees (temperature) to 0~180 degrees (angle)
        servoAngle(finalAngle);
        _delay_ms(500);
    }
} 